# -*- coding: utf-8 -*-
#
##########################################################################
# pyeole.diagnose - Diagnostic tools for EOLE
# Copyright © 2013-2014 Pôle de compétences EOLE <eole@ac-dijon.fr>
#
# License CeCILL:
#  * in french: http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
#  * in english http://www.cecill.info/licences/Licence_CeCILL_V2-en.html
##########################################################################

from _socket import timeout as socket_timeout

from urllib2 import urlopen
from urllib2 import Request
from urllib2 import HTTPError
from urllib2 import URLError

import time, datetime
import os
from os import environ
from os.path import isfile
from commands import getstatusoutput

import logging

from creole.client import CreoleClient
from pyeole.process import system_code

log = logging.getLogger(__name__)

OK=0
NOK=1
NOT_AVAILABLE=2
CACHE_IPTABLES = '/etc/eole/iptables'
CACHE_IPSET = '/etc/eole/ipset'
BIN_IPTABLES_SAVE = '/sbin/iptables-save'
BIN_IPSET = '/usr/sbin/ipset'
global activer_agregation
activer_agregation = None
PIDFILE='/var/run/clamav/clamd.pid'
SOCKETFILE='/run/clamav/clamd'

try:
    import pyclamd
except:
    pyclamd = None
CLAMD_CONTAINER = '/etc/eole/clamd-reload.cnf'
FRESHCLAM_FILE = '/var/log/clamav/freshclam-status.log'

def test_http(url, use_proxy=False):
    """Testing if an HTTP URL is reachable

    :param url: URL to check
    :type url: `str`
    :param use_proxy: let urllib2 use proxy setting from environment
    :type use_proxy: `bool`
    :return: if an HTTP request success on the :data:`url`
    :rtype: `bool`

    """
    log.debug(u'Test if {0} is reachable'.format(url))
    http_proxy = None
    if not use_proxy:
        try:
            http_proxy = environ.pop('http_proxy')
        except KeyError:
            pass

    try:
        req = Request(url)
        reponse = urlopen(req)
        ret = True
    except (HTTPError, URLError, socket_timeout):
        ret = False

    if http_proxy is not None:
        # restore environment
        environ.update({'http_proxy': http_proxy})

    return ret


def compare_iptables():
    global activer_agregation
    if not isfile(CACHE_IPTABLES) or not isfile(BIN_IPTABLES_SAVE):
        return(NOT_AVAILABLE)
    code, out = getstatusoutput(BIN_IPTABLES_SAVE)
    if code == 0:
        cache_iptables = set(open(CACHE_IPTABLES, 'r').read().splitlines())
        new_iptables = set(out.splitlines())
        for not_present in cache_iptables - new_iptables:
            if activer_agregation is None:
                client = CreoleClient()
                activer_agregation = client.get_creole('activer_agregation', 'non')
            if not_present[0] not in [':', '#'] and (activer_agregation == 'non' or "SNAT" not in not_present):
                return(NOK)
        #test les chaines et les politiques par défaut
        new_policy = set()
        for new in new_iptables:
            if new.startswith(':'):
                new_policy.add(new.split('[')[0])
        cache_policy = set()
        for cache in cache_iptables:
            if cache.startswith(':'):
                cache_policy.add(cache.split('[')[0])
        if cache_policy - new_policy:
            return(NOK)

    else:
        print('impossible de lancer {0}'.format(BIN_IPTABLES_SAVE))
        return(NOK)
    return(OK)


def compare_ipset():
    if not isfile(CACHE_IPSET) or not isfile(BIN_IPSET):
        return(NOT_AVAILABLE)
    code, out = getstatusoutput(BIN_IPSET + ' save')
    if code == 0:
        cache_iptables = set(open(CACHE_IPSET, 'r').read().splitlines())
        new_iptables = set(out.splitlines())
        if cache_iptables - new_iptables != set([]):
            return(NOK)
    else:
        print('impossible de lancer {0}'.format(BIN_IPSET))
        return(NOK)
    return(OK)


def test_freshclam():
    def _parse_file():
        if ret['STATUS'] == 0:
            now = datetime.datetime.now()
            last = datetime.datetime.utcfromtimestamp(ret['DATE'])
            one_week_ago = datetime.timedelta(weeks=1)
            last_update = time.strftime("%d %b %Y %H:%M:%S", time.localtime(ret['DATE']))
            if now - last < one_week_ago:
                status = 'On'
                msg = 'Base antivirale mise à jour le %s' % last_update
            else:
                status = 'Off'
                msg = "Base antivirale mise à jour le %s, soit il y a plus d'une semaine" % last_update
        else:
            status = 'Off'
            if ret['STATUS'] == 1:
                msg = "Impossible de mettre à jour la base de signature"
            else:
                msg = "Base antivirale à jour, mais freshclam n'a pas réussi à relancer clamd à l'issue de la mise jour"
        return status, msg

    ret = {}
    if isfile(FRESHCLAM_FILE) and os.stat(FRESHCLAM_FILE).st_size != 0:
        try:
            fh = open(FRESHCLAM_FILE)
            for line in fh.readlines():
                if line.startswith('STATUS='):
                    ret['STATUS'] = int(line.split('"')[1])
                elif line.startswith('DATE='):
                    ret['DATE'] = float(line.split('"')[1])
            status, msg = _parse_file()
        except Exception, e:
            status = 'Off'
            msg = "Impossible de lire %s : %s" % (FRESHCLAM_FILE, e)
    else:
        status = ''
        msg = "Le fichier %s n'existe pas ou est vide" % FRESHCLAM_FILE
    return {'msg': msg, 'status': status}


def test_pyclamd(only_ping=True):
    try:
        if pyclamd is None:
            return 1
        if not isfile(PIDFILE):
            return 1
        pyclamd.init_unix_socket(filename=SOCKETFILE)
        pyclamd.ping()
        if only_ping:
            return 0
        pyclamd.reload()
        return 0
    except:
        print 'Could not ping or reload clamd'
        return 1


def test_clamd():
    if isfile(CLAMD_CONTAINER):
        try:
            for line in open(CLAMD_CONTAINER, 'r').readlines():
                if line.startswith('CLAMD_CONTAINER='):
                    containers = line.split('"')[1].split()
                    if containers == []:
                        #aucun service clam n'est activé
                        return 1
                    for container in containers:
                        if not container.isalnum():
                            msg = 'a container has an invalid name'
                            #fait une erreur dans le diagnose
                            #log.error(msg + ' ' + container)
                            raise Exception('a container has an invalid name')
                    break
        except Exception, err:
            #fait une erreur dans le diagnose
            #log.error('cannot read {0} file: {1}'.format(CLAMD_CONTAINER, err), exc_info=True)
            import traceback
            traceback.print_exc()
            return 1
        else:
            for container in containers:
                code = system_code(['/usr/bin/clamd-eole', '-p'], container)
                if code != 0:
                    print "hu?"
                    return 1
            return 0
    else:
        return ''

